#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Prepare figures for ME-MP2RAGE manuscript
"""

#    Copyright (C) 2015 Riccardo Metere <metere@cbs.mpg.de>
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# ======================================================================
# :: Future Imports
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

# ======================================================================
# :: Python Standard Library Imports
import os  # Miscellaneous operating system interfaces
# import shutil  # High-level file operations
# import math  # Mathematical functions
import time  # Time access and conversions
import datetime  # Basic date and time types
# import operator  # Standard operators as functions
# import collections  # High-performance container datatypes
import argparse  # Parser for command-line options, arguments and subcommands
import itertools  # Functions creating iterators for efficient looping
# import subprocess  # Subprocess management
# import multiprocessing  # Process-based parallelism
import csv  # CSV File Reading and Writing [CSV: Comma-Separated Values]
import json  # JSON encoder and decoder [JSON: JavaScript Object Notation]
import string  # Common string operations

# :: External Imports
import numpy as np  # NumPy (multidimensional numerical arrays library)
# import scipy as sp  # SciPy (signal and image processing library)
import matplotlib as mpl  # Matplotlib (2D/3D plotting library)
# import sympy as sym  # SymPy (symbolic CAS library)
# import PIL  # Python Image Library (image manipulation toolkit)
import nibabel as nib  # NiBabel (NeuroImaging I/O Library)
# import nipy  # NiPy (NeuroImaging in Python)
# import nipype  # NiPype (NiPy Pipelines and Interfaces)

# :: External Imports Submodules
import matplotlib.pyplot as plt  # Matplotlib's pyplot: MATLAB-like syntax
# import mayavi.mlab as mlab  # Mayavi's mlab: MATLAB-like syntax
# import scipy.optimize  # SciPy: Optimization Algorithms
# import scipy.integrate  # SciPy: Integrations facilities
# import scipy.constants  # SciPy: Mathematal and Physical Constants
# import scipy.ndimage  # SciPy: ND-image Manipulation

import mri_tools.base as mrb
import mri_tools.utils as mru
# import mri_tools.geometry as mrg
# import mri_tools.plot as mrp
import mri_tools.input_output as mrio
# import mri_tools.correlation as mrl
# import mri_tools.registration as mrr

from mri_tools import INFO
from mri_tools import VERB_LVL
from mri_tools import D_VERB_LVL


# =====================================================================
def prepare_figures(
        dirpath,
        subpath,
        sources,
        dpi,
        img_type):
    """
    Prepare figures for the publication
    """
    # plotting
    widths = [1.30, 1.30, 1.30]
    heights = [1.0]
    num_row = len(heights)
    num_col = len(widths)
    figsize_factor = (3.5,) * 2
    save_path = mrb.change_ext('registration__all', img_type)

    fig = plt.figure(
        figsize=(
            figsize_factor[0] * sum(widths),
            figsize_factor[1] * sum(heights)))

    gs = mpl.gridspec.GridSpec(
        num_row, num_col, width_ratios=widths, height_ratios=heights)

    axs = [plt.subplot(gs[n]) for n in range(num_row * num_col)]
    axs = np.array(axs).reshape(num_row, num_col)

    j = 0
    mappings = 'T1', 'T2S', 'CHI'
    for i, mapping in enumerate(mappings):
        data = sources[mapping]
        mask_path = os.path.join(dirpath, mapping.lower(), 'mask.nii.gz')
        ref_path = os.path.join(dirpath, mapping.lower(), subpath, data['ref'])
        test_path = os.path.join(dirpath, mapping.lower(), subpath,
                                 data['test'])
        letter = string.ascii_lowercase[j * num_col + i]
        mrio.plot_histogram2d(
            ref_path, test_path, mask_path, mask_path,
            ax=axs[j, i], use_new_figure=False,
            array_interval=data['val_interval'],
            colorbar_opts={}, hist_val_interval=(0, 1000),
            bisector=':k', bins=400, scale='linear', ticks_limit=5,
            stats_opts={
                'mask_inf': True, 'mask_nan': True,
                'mask_vals': [0.0]},
            title='({}) 2D Histogram / ${}$'.format(letter, data['name']),
            labels=(
                '{} ${}$ / {}'.format(
                    'unmodified', data['name'], data['val_units']),
                '{} ${}$ / {}'.format(
                    'registered', data['name'], data['val_units'])))

    # fig.subplots_adjust(
    #     left=0.0, bottom=0.0, right=1.0, top=1.0,
    #     wspace=None, hspace=None)
    gs.tight_layout(fig, rect=[0.0, 0.0, 1.0, 1.0])
    fig.savefig(save_path, dpi=int(dpi))
    plt.close(fig)
    print(save_path)


# ======================================================================
def handle_arg():
    """
    Handle command-line application arguments.
    """
    # :: Create Argument Parser
    arg_parser = argparse.ArgumentParser(
        description=__doc__,
        epilog='v.{} - {}\n{}'.format(
            INFO['version'], ', '.join(INFO['authors']), INFO['license']),
        formatter_class=argparse.RawDescriptionHelpFormatter)
    # :: Add POSIX standard arguments
    arg_parser.add_argument(
        '--ver', '--version',
        version='%(prog)s - ver. {}\n{}\n{} {}\n{}'.format(
            INFO['version'],
            next(line for line in __doc__.splitlines() if line),
            INFO['copyright'], ', '.join(INFO['authors']),
            INFO['notice']),
        action='version')
    arg_parser.add_argument(
        '-v', '--verbose',
        action='count', default=D_VERB_LVL,
        help='increase the level of verbosity [%(default)s]')
    # :: Add additional arguments
    arg_parser.add_argument(
        '-f', '--force',
        action='store_true',
        help='force new processing [%(default)s]')
    arg_parser.add_argument(
        '-a', '--variant',
        type=int, default=0,
        help='select variant for figure creation [%(default)s]')
    arg_parser.add_argument(
        '-p', '--dpi',
        type=float, default=50,
        help='export resolution in DPI [%(default)s]')
    arg_parser.add_argument(
        '-t', '--img_type',
        default='eps',
        help='export image file type [%(default)s]')
    return arg_parser


# ======================================================================
def main():
    # :: handle program parameters
    arg_parser = handle_arg()
    args = arg_parser.parse_args()
    # :: print debug info
    if args.verbose == VERB_LVL['debug']:
        arg_parser.print_help()
        print()
        print('II:', 'Parsed Arguments:', args)

    kwargs = {
        'dirpath': '/nobackup/isar2/cache/me-mp2rage/misreg/reg',
        'subpath': 'msk',
        'sources': {
            'T1': {
                'name': 'T_1',
                'val_interval': (700, 4000),
                'val_units': 'ms',
                'ref_seq': 'MP2RAGE',
                'ref': 's013__MP2RAGE_bw=280,noise=0.0,off=0.0,res=0.6,'
                       'rot=0.0,te=2.35,tr=5__T1.nii.gz',
                'test': 's013__MP2RAGE_bw=280,noise=0.0,off=0.0,res=0.6,'
                        'rot=10.0,te=2.35,tr=5__T1.nii.gz',

            },
            'T2S': {
                'name': 'T_2^*',
                'val_interval': (1, 80),
                'val_units': 'ms',
                'ref_seq': 'FLASH',
                'ref': 's002__ME-FLASH_fa=11,noise=0.0,off=0.0,res=0.6,'
                       'rot=0.0,tr=31__T2S.nii.gz',
                'test': 's002__ME-FLASH_fa=11,noise=0.0,off=0.0,res=0.6,'
                        'rot=10.0,tr=31__T2S.nii.gz',
            },
            'CHI': {
                'name': '\chi',
                'val_interval': (-99, +99),
                'val_units': 'ppb',
                'ref_seq': 'FLASH',
                'ref': 'QSM_noise=0.0,off=0.0,rot=0.0,te=15,'
                       'firstflash__CHI.nii.gz',
                'test': 'QSM_noise=0.0,off=0.0,rot=10.0,te=15,'
                        'firstflash__CHI.nii.gz',
            }
        },
        'dpi': args.dpi,
        'img_type': args.img_type
    }

    if args.variant > 0:
        eval('prepare_figures' + str(args.variant) + '(**kwargs)')
    else:
        prepare_figures(**kwargs)

    mrb.elapsed('create plots')
    if args.verbose > VERB_LVL['low']:
        mrb.print_elapsed()


# ======================================================================
if __name__ == '__main__':
    main()
